"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var oly_1 = require("oly");
var errors_1 = require("../constants/errors");
/**
 * Exception wrapper of AxiosError. <br/>
 *
 * ### Fields
 *
 * | | type | default  | description |
 * |--|--|--|--|
 * | status | number | -1 | HTTP status |
 * | body | any | undefined | HTTP response body |
 * | cause | AxiosError | N/A | axios error |
 *
 * ```ts
 * try {
 *   await http.get("/");
 * } catch(e) {
 *   if(e instanceof HttpClientException) {
 *
 *     if (e.status === 401) {
 *       // ...
 *     }
 *   }
 * }
 * ```
 *
 * ### Follow HttpServerException
 *
 * ```ts
 * try {
 *   await http.get("/");
 * } catch(e) {
 *   if (e instanceof HttpClientException) {
 *
 *     if (e.isHttpServerException()) {
 *        (e as HttpClientException<HttpServerException>)
 *         .body
 *         .message; // ...
 *     }
 *   }
 * }
 * ```
 */
var HttpClientException = /** @class */ (function (_super) {
    tslib_1.__extends(HttpClientException, _super);
    function HttpClientException(error) {
        var _this = _super.call(this, error) || this;
        /**
         * Status of the failed request.
         */
        _this.status = -1;
        if (!error.response) {
            // Error network
            _this.message = error.message;
            return _this;
        }
        if (_this.isHttpServerException()) {
            // Error come from our Server with an HttpServerException
            _this.message = errors_1.olyHttpErrors.requestHasFailedWithMessage(error.config.method, error.config.url, error.response.status, error.response.data.message);
            _this.status = error.response.data.status;
            _this.body = error.response.data;
        }
        else {
            // Error come from somewhere else
            if (typeof error.response.data === "string") {
                _this.message = errors_1.olyHttpErrors.requestHasFailedWithMessage(error.config.method, error.config.url, error.response.status, error.response.data);
            }
            else {
                _this.message = errors_1.olyHttpErrors.requestHasFailed(error.config.method, error.config.url, error.response.status);
            }
            _this.status = error.response.status;
            _this.body = error.response.data;
        }
        return _this;
    }
    HttpClientException.prototype.toJSON = function () {
        return tslib_1.__assign({}, _super.prototype.toJSON.call(this), { body: this.body, status: this.status });
    };
    HttpClientException.prototype.isHttpServerException = function (withName) {
        if (!(!!this.cause
            && !!this.cause.response
            && !!this.cause.response.data
            && !!this.cause.response.data.status
            && !!this.cause.response.data.name
            && !!this.cause.response.data.message)) {
            return false;
        }
        if (!withName) {
            return true;
        }
        return this.cause.response.data.name === withName;
    };
    return HttpClientException;
}(oly_1.Exception));
exports.HttpClientException = HttpClientException;
//# sourceMappingURL=HttpClientException.js.map