"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var http_1 = require("http");
var Koa = require("koa");
var oly_1 = require("oly");
var _1 = require("../");
var middlewares_1 = require("../middlewares");
/**
 * Default http server provider
 */
var HttpServerProvider = /** @class */ (function () {
    function HttpServerProvider() {
        /**
         * Set hostname of the http server.
         */
        this.host = "127.0.0.1";
        /**
         * Set port of the http server.
         */
        this.port = 3000;
        /**
         * Koa application.
         * Http Server is provided with Koa.
         */
        this.app = new Koa();
    }
    Object.defineProperty(HttpServerProvider.prototype, "hostname", {
        /**
         * Hostname getter.
         * Protocol is forced to http as we only use Http Server.
         */
        get: function () {
            return "http://" + this.host + ":" + this.port;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Configure middleware before onStart().
     *
     * @param middleware
     */
    HttpServerProvider.prototype.use = function (middleware) {
        this.app.use(middleware);
        return this;
    };
    /**
     * Start the http server.
     * Listen new connections.
     */
    HttpServerProvider.prototype.onStart = function () {
        var _this = this;
        // override koa context with our forked kernel
        // we fork kernel to protect the main layer
        this
            .use(middlewares_1.context(this.kernel))
            .useLogger()
            .useHelmet();
        // start server
        return new Promise(function (resolve, reject) {
            _this.http = _this.createServer();
            _this.http.listen(_this.port, _this.host, function (err) {
                if (err) {
                    return reject(err);
                }
                _this.logger.info("listening on " + _this.hostname);
                resolve();
            });
        });
    };
    /**
     * Stop http server.
     * We don't wait the callback.
     */
    HttpServerProvider.prototype.onStop = function () {
        var _this = this;
        // stop server
        this.logger.info("stop server");
        return new Promise(function (resolve, reject) {
            return _this.http.shutdown(function (err) {
                err ? reject(err) : resolve();
            });
        });
    };
    /**
     * Use helmet middleware.
     *
     * @overridable
     */
    HttpServerProvider.prototype.useHelmet = function () {
        return this.use(_1.helmet());
    };
    /**
     * Use a http logger middleware.
     *
     * @overridable
     */
    HttpServerProvider.prototype.useLogger = function () {
        this.use(function (ctx, next) {
            var logger = ctx.kernel.get(oly_1.Logger).as("HttpServer");
            var now = Date.now();
            logger.info("--> " + ctx.method + " " + ctx.path);
            logger.trace("incoming request", ctx.request.toJSON());
            return next().then(function () {
                var time = Date.now() - now;
                if (ctx.status === 500 && typeof ctx.body === "object") {
                    logger.error("internal error", ctx.body);
                }
                logger.info("<-- " + ctx.method + " " + ctx.path + " " + ctx.status + " - " + time + "ms");
                logger.trace("response", ctx.response.toJSON());
            });
        });
        return this;
    };
    /**
     * @overridable
     */
    HttpServerProvider.prototype.createServer = function () {
        return require("http-shutdown")(http_1.createServer(this.app.callback()));
    };
    tslib_1.__decorate([
        oly_1.env("HTTP_SERVER_HOST"),
        tslib_1.__metadata("design:type", String)
    ], HttpServerProvider.prototype, "host", void 0);
    tslib_1.__decorate([
        oly_1.env("HTTP_SERVER_PORT"),
        tslib_1.__metadata("design:type", Number)
    ], HttpServerProvider.prototype, "port", void 0);
    tslib_1.__decorate([
        oly_1.inject,
        tslib_1.__metadata("design:type", oly_1.Kernel)
    ], HttpServerProvider.prototype, "kernel", void 0);
    tslib_1.__decorate([
        oly_1.inject,
        tslib_1.__metadata("design:type", oly_1.Logger)
    ], HttpServerProvider.prototype, "logger", void 0);
    tslib_1.__decorate([
        oly_1.state,
        tslib_1.__metadata("design:type", Koa)
    ], HttpServerProvider.prototype, "app", void 0);
    tslib_1.__decorate([
        oly_1.state,
        tslib_1.__metadata("design:type", http_1.Server)
    ], HttpServerProvider.prototype, "http", void 0);
    return HttpServerProvider;
}());
exports.HttpServerProvider = HttpServerProvider;
//# sourceMappingURL=HttpServerProvider.js.map